package calculator
import expression._

// Mettre compute, variables et simplify1 et simplify2 (appel des fonctions cachées) dans calculator
object Calculator {
  def compute(e: Expression, env: Map[String, Int]): Option[Int] = {
    e match {
      case Constant(i) => Some(i)
      case Sum(e1, e2) => {
        (compute(e1, env), compute(e2, env)) match {
          case (Some(v1), Some(v2)) => Some(v1 + v2)
          case _                    => None
        }
      }
      case Sub(e1, e2) => {
        (compute(e1, env), compute(e2, env)) match {
          case (Some(v1), Some(v2)) => Some(v1 - v2)
          case _                    => None
        }
      }
      case Variable(x) =>
        if (env.contains(x)) Some(env(x)) else None
    }
  }

  def variables(e: Expression): Set[String] = {
    e match {
      case Variable(x) => Set(x)
      case Constant(a) => Set()
      case Sub(e1, e2) => variables(e1) ++ variables(e2)
      case Sum(e1, e2) => variables(e1) ++ variables(e2)
    }
  }

  val simp1 = new simplifier.Simplifier1()
  val simp2 = new simplifier.Simplifier2()

  def simplify1(e: Expression): Expression = simp1.simplify(e)
  def simplify2(e: Expression): (Expression, List[(Int, Int)]) = {
    val r: Result = simp2.simplify(e)
    (r.getExpression(), r.getPath())
  }

}
