package expression

import scala.util.parsing.combinator.syntactical.StdTokenParsers
import scala.util.parsing.combinator.lexical.StdLexical

case class BadExpression(s: String) extends Exception

object Parser extends StdTokenParsers {

  def parseString(s: String): Expression = {
    val parseResult = expr(
      new lexical.Scanner(s)
    )
    if (!parseResult.successful) throw new BadExpression(s)
    else parseResult.get
  }

  type Tokens = StdLexical

  import lexical.{Keyword, NumericLit, StringLit, Identifier}

  val lexical = new StdLexical

  lexical.delimiters ++= List("(", ")", "+", "-")

  def expr: Parser[Expression] = sumExp | subExp | unExp
  def factor: Parser[Expression] = "(" ~> expr <~ ")"

  def sumExp: Parser[Expression] =
    (unExp ~ "+" ~ expr ^^ { case x ~ _ ~ y => new Sum(x, y) }
      | unExp ~ "+" ~ expr ^^ { case x ~ _ ~ y => new Sum(x, y) })

  def subExp: Parser[Expression] =
    (unExp ~ "-" ~ expr ^^ { case x ~ _ ~ y => new Sub(x, y) }
      | unExp ~ "-" ~ expr ^^ { case x ~ _ ~ y => new Sub(x, y) })

  def unExp: Parser[Expression] = integerExp | callVar | factor | negInt

  def negInt: Parser[Expression] = "-" ~ numericLit ^^ { case _ ~ s =>
    Constant(-s.toInt)
  }
  def integerExp: Parser[Expression] = numericLit ^^ { case s =>
    Constant(s.toInt)
  }

  def callVar: Parser[Expression] = ident ^^ { case s => Variable(s) }
  def operator: Parser[String] = "-" | "+" ^^ { case s => s }
}
